// COMPSCI375_Tutorial2.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <cv.h>
#include <cxcore.h>
#include <highgui.h>

#include <iostream>

void colourFlow(IplImage * flowImage, IplImage * colourImage)
{
	/*
	!colourImage should be created as follows!
	IplImage * colourImage =
		cvCreateImage(cvGetSize(flowImage),IPL_DEPTH_8U,3);
	*/
	if (flowImage->depth != IPL_DEPTH_32F)
	{
		printf("Not a flow image");
		return;
	}

	// Initialise values
	double maxValue = -1e10;
	double minValue = 1e10;

	CvScalar s;
	int i, j;

	// Find max and min value
	for (i = 0 ; i < flowImage -> height ; i++)
	{
		for (j = 0 ; j < flowImage -> width ; j++)
		{
			s=cvGet2D(flowImage,i,j); // get the (i,j) pixel value
			if (minValue > s.val[0] )
				minValue = s.val[0];
			if (maxValue < s.val[0])
				maxValue = s.val[0];
		}
	}
	printf("%f,%f\n",minValue,maxValue);
	
	double scaleValue;
	CvScalar flowValue;

	// Create colour image
	for (i = 0 ; i < flowImage -> height ; i++)
	{
		for (j = 0 ; j < flowImage -> width ; j++)
		{
			// get the (i,j) pixel value
			flowValue=cvGet2D(flowImage,i,j); 
			// Set colour channels
			// max negative = green
			// zero = black
			// max posative = red
			if (flowValue.val[0] > 0)
			{
				scaleValue = flowValue.val[0] / maxValue;
				s.val[2] = 255 * scaleValue;	// set R
				s.val[1] = 0;					// set G
			}
			else
			{
				scaleValue = flowValue.val[0] / minValue;
				s.val[1] = 255 * scaleValue;	// set G
				s.val[2] = 0;					// set R
			}

			// assign colour to colour image
			s.val[0] = 0;		
			cvSet2D(colourImage,i,j,s);
		}
	}

}

void runOpticalFlow(IplImage * current, IplImage * previous)
{
	// Runs Lucas-Kanade optical flow

	// Error checks
	if ( current->width != previous->width ||
		 current->height != previous->height   )
	{
		printf("Size of previous and current image not the same\n");
		return;
	}

	// Set aside memory for grey scale image
	IplImage * gCurrent = 
		cvCreateImage( cvGetSize(current),
		               IPL_DEPTH_8U,
					   1);
	IplImage * gPrevious = 
		cvCreateImage( cvGetSize(previous),
		               IPL_DEPTH_8U,
					   1);
	
	// Set aside memory for the flow images
	IplImage * xFlow = 
		cvCreateImage( cvGetSize(previous),
		               IPL_DEPTH_32F,
					   1);
	IplImage * yFlow = 
		cvCreateImage( cvGetSize(previous),
		               IPL_DEPTH_32F,
					   1);

	// Create grey scale images from originals
	cvCvtColor(current,gCurrent,CV_BGR2GRAY);
	cvCvtColor(previous,gPrevious,CV_BGR2GRAY);

	// Calculate Optical Flow
	cvCalcOpticalFlowLK(gPrevious,gCurrent,cvSize(15,15),
		                xFlow,yFlow);

	// Set aside memory for colour image to display xFlow
	IplImage * colourImage =
		cvCreateImage(cvGetSize(xFlow),IPL_DEPTH_8U,3);

	// Get colour image from flow
	colourFlow(xFlow, colourImage);

	// Display images
	cvNamedWindow( "xFlow",CV_WINDOW_AUTOSIZE);
    cvShowImage("xFlow", colourImage );
	
	// Clear memory
	cvReleaseImage(&gCurrent);
	cvReleaseImage(&gPrevious);
	cvReleaseImage(&xFlow);
	cvReleaseImage(&yFlow);
	cvReleaseImage(&colourImage);
}

int _tmain(int argc, _TCHAR* argv[])
{
	// Create windows for previous and current frame
	cvNamedWindow( "t-1",CV_WINDOW_AUTOSIZE);
	cvNamedWindow( "t",CV_WINDOW_AUTOSIZE);
	
	//Create images
	IplImage * image = NULL;
	IplImage * oldImage = NULL;

	// Constants for image sequence
	const int No_Of_Frame = 6;
	char buffer[100];

	// Initialise sequence
	int i = 1;
	while(i < No_Of_Frame && i > 0)
	{ 
		// The path and by using sprintf 
		// we can add the image number instead of the %i
		sprintf(buffer,"bmps\\%i.bmp",i);

		// load image
		image = cvLoadImage( buffer );

		// show images
		cvShowImage("t", image );
		if (oldImage != NULL)
		{
			cvShowImage("t-1", oldImage );
			runOpticalFlow(image, oldImage);
		}

		// cvWaitKey work as a timer or the framerate 
		// for showing our sequence
		int keyPressed = cvWaitKey(0);

		// Go backward if you push backspace
		if (keyPressed == 8)
			i--;
		// Go forward if any other key pressed		
		else
			i++;

		printf("%i \n",keyPressed);

		// Release memory
		if (oldImage != NULL)
			cvReleaseImage(&oldImage);
		
		// Copy current image to previous image
		oldImage = cvCloneImage(image);
		
		// Release memory
		cvReleaseImage(&image);
	}

	// Release windows
	cvDestroyAllWindows();
	return 0;
}